#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "help_mp.h"

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

#include "codec-cfg.h"

#include "../libvo/img_format.h"

#include "stream.h"
#include "demuxer.h"
#include "stheader.h"

#include "vd.h"
#include "../postproc/vf.h"
#include "vd_msg.h"

extern vd_functions_t mpcodecs_vd_null;
extern vd_functions_t mpcodecs_vd_ffmpeg;
extern vd_functions_t mpcodecs_vd_dshow;
extern vd_functions_t mpcodecs_vd_vfw;
extern vd_functions_t mpcodecs_vd_vfwex;
extern vd_functions_t mpcodecs_vd_divx4;
extern vd_functions_t mpcodecs_vd_raw;
extern vd_functions_t mpcodecs_vd_xanim;
extern vd_functions_t mpcodecs_vd_fli;
extern vd_functions_t mpcodecs_vd_nuv;
extern vd_functions_t mpcodecs_vd_mpng;
extern vd_functions_t mpcodecs_vd_ijpg;
extern vd_functions_t mpcodecs_vd_libmpeg2;
extern vd_functions_t mpcodecs_vd_xvid;
extern vd_functions_t mpcodecs_vd_mpegpes;
extern vd_functions_t mpcodecs_vd_huffyuv;
extern vd_functions_t mpcodecs_vd_xanim;
extern vd_functions_t mpcodecs_vd_real;
extern vd_functions_t mpcodecs_vd_dmo;
extern vd_functions_t mpcodecs_vd_qtvideo;
extern vd_functions_t mpcodecs_vd_theora;

vd_functions_t* mpcodecs_vd_drivers[] = {
        &mpcodecs_vd_null,
        &mpcodecs_vd_ffmpeg,
        &mpcodecs_vd_dshow,
        &mpcodecs_vd_vfw,
        &mpcodecs_vd_vfwex,
        &mpcodecs_vd_divx4,
        &mpcodecs_vd_raw,
        &mpcodecs_vd_nuv,
        &mpcodecs_vd_libmpeg2,
        &mpcodecs_vd_xvid,
        &mpcodecs_vd_mpegpes,
        &mpcodecs_vd_huffyuv,
        &mpcodecs_vd_xanim,
        &mpcodecs_vd_real,
        &mpcodecs_vd_dmo,
        &mpcodecs_vd_qtvideo,
        &mpcodecs_vd_theora,
	NULL
};

#include "libvo/video_out.h"

extern vd_functions_t* mpvdec; // FIXME!

int mpcodecs_config_vo(sh_video_t *sh, int w, int h, void *tune){
    int i,j;
    unsigned int out_fmt=0;
    int screen_size_x=0;//SCREEN_SIZE_X;
    int screen_size_y=0;//SCREEN_SIZE_Y;
    vf_instance_t* vf=sh->vfilter,*sc=NULL;
    int palette=0;

    if(!(sh->disp_w && sh->disp_h))
        MSG_WARN(
            "VDec: driver %s didn't set sh->disp_w and sh->disp_h, trying to workaround!\n"
	    ,sh->codec->codec_name);
    /* XXX: HACK, if sh->disp_* aren't set,
     * but we have w and h, set them :: atmos */
    if(!sh->disp_w && w)
        sh->disp_w=w;
    if(!sh->disp_h && h)
        sh->disp_h=h;

    MSG_V("VDec: vo config request - %d x %d\n",
	w,h);

csp_again:
    // check if libvo and codec has common outfmt (no conversion):
    j=-1;
    for(i=0;i<CODECS_MAX_OUTFMT;i++){
	out_fmt=sh->codec->outfmt[i];
	if(out_fmt==(signed int)0xFFFFFFFF) continue;
	vo_flags=vf->query_format(vf,out_fmt,w,h);
	MSG_DBG2("vo_debug: query(%s) returned 0x%X (i=%d) \n",vo_format_name(out_fmt),vo_flags,i);
	if((vo_flags&2) || (vo_flags && j<0)){
	    // check (query) if codec really support this outfmt...
	    if(mpvdec->control(sh,VDCTRL_QUERY_FORMAT,&out_fmt)==CONTROL_FALSE)
		continue;
	    j=i; if(vo_flags&2) break;
	} else
	if(!palette && !(vo_flags&3) && (out_fmt==IMGFMT_RGB8||out_fmt==IMGFMT_BGR8)){
	    sh->outfmtidx=j; // pass index to the control() function this way
	    if(mpvdec->control(sh,VDCTRL_QUERY_FORMAT,&out_fmt)!=CONTROL_FALSE)
		palette=1;
	}
    }
    if(j<0){
	// TODO: no match - we should use conversion...
	if(strcmp(vf->info->name,"fmtcvt") && palette!=1){
	    MSG_INFO("Can't find colorspace\n");
	    sc=vf=vf_open_filter(vf,sh,"fmtcvt",NULL);
	    goto csp_again;
	} else
	if(palette==1){
	    MSG_V("vd: Trying -vf palette...\n");
	    palette=-1;
	    vf=vf_open_filter(vf,sh,"palette",NULL);
	    goto csp_again;
	} else 
	{ // sws failed, if the last filter (vf_vo) support MPEGPES try to append vf_lavc
	     vf_instance_t* vo, *vp = NULL, *ve;
	     // Remove the scale filter if we added it ourself
	     if(vf == sc) {
	       ve = vf;
	       vf = vf->next;
	       vf_uninit_filter(ve);
	     }
	     // Find the last filter (vf_vo)
	     for(vo = vf ; vo->next ; vo = vo->next)
	       vp = vo;
	}
	MSG_WARN(MSGTR_VOincompCodec);
	sh->vfilter_inited=-1;
	return 0;	// failed
    }
    out_fmt=sh->codec->outfmt[j];
    sh->outfmtidx=j;
    sh->vfilter=vf;

    // autodetect flipping
    if(flip==-1){
	flip=0;
	if(sh->codec->outflags[j]&CODECS_FLAG_FLIP)
	    if(!(sh->codec->outflags[j]&CODECS_FLAG_NOFLIP))
		flip=1;
    }
    if(vo_flags&VFCAP_FLIPPED) flip^=1;
    if(flip && !(vo_flags&VFCAP_FLIP)){
	// we need to flip, but no flipping filter avail.
	sh->vfilter=vf=vf_open_filter(vf,sh,"mirror","x");
    }

    // time to do aspect ratio corrections...

  if(movie_aspect>-1.0) sh->aspect = movie_aspect; // cmdline overrides autodetect
  if(opt_screen_size_x||opt_screen_size_y){
    screen_size_x = opt_screen_size_x;
    screen_size_y = opt_screen_size_y;
    if(!vidmode){
     if(!screen_size_x) screen_size_x=SCREEN_SIZE_X;
     if(!screen_size_y) screen_size_y=SCREEN_SIZE_Y;
     if(screen_size_x<=8) screen_size_x*=sh->disp_w;
     if(screen_size_y<=8) screen_size_y*=sh->disp_h;
    }
  } else {
    // check source format aspect, calculate prescale ::atmos
    screen_size_x=sh->disp_w;
    screen_size_y=sh->disp_h;
    if(screen_size_xy>=0.001){
     if(screen_size_xy<=8){
       // -xy means x+y scale
       screen_size_x*=screen_size_xy;
       screen_size_y*=screen_size_xy;
     } else {
       // -xy means forced width while keeping correct aspect
       screen_size_x=screen_size_xy;
       screen_size_y=screen_size_xy*sh->disp_h/sh->disp_w;
     }
    }
    if(sh->aspect>0.01){
      int w;
      MSG_V("Movie-Aspect is %.2f:1 - prescaling to correct movie aspect.\n",
             sh->aspect);
      w=(int)((float)screen_size_y*sh->aspect); w+=w%2; // round
      // we don't like horizontal downscale || user forced width:
      if(w<screen_size_x || screen_size_xy>8){
        screen_size_y=(int)((float)screen_size_x*(1.0/sh->aspect));
        screen_size_y+=screen_size_y%2; // round
        if(screen_size_y<sh->disp_h) // Do not downscale verticaly
            screen_size_y=sh->disp_h;
      } else screen_size_x=w; // keep new width
    } else {
      MSG_V("Movie-Aspect is undefined - no prescaling applied.\n");
    }
  }

    MSG_V("video_out->init(%dx%d->%dx%d,flags=%d,'%s',%s)\n",
                      sh->disp_w,sh->disp_h,
                      screen_size_x,screen_size_y,
                      fullscreen|(vidmode<<1)|(softzoom<<2)|(flip<<3),
                      "MPlayerXP",vo_format_name(out_fmt));

    if(vf->config(vf,sh->disp_w,sh->disp_h,
                      screen_size_x,screen_size_y,
                      fullscreen|(vidmode<<1)|(softzoom<<2)|(flip<<3),
                      out_fmt,tune)==0){
	MSG_WARN(MSGTR_CannotInitVO);
	sh->vfilter_inited=-1;
	return 0;
    }
    return 1;
}

// mp_imgtype: buffering type, see mp_image.h
// mp_imgflag: buffer requirements (read/write, preserve, stride limits), see mp_image.h
// returns NULL or allocated mp_image_t*
// Note: buffer allocation may be moved to mpcodecs_config_vo() later...
mp_image_t* mpcodecs_get_image(sh_video_t *sh, int mp_imgtype, int mp_imgflag, int x, int y,int w, int h){
  mp_image_t* mpi=vf_get_slice(sh->vfilter,sh->codec->outfmt[sh->outfmtidx],mp_imgtype,mp_imgflag,x,y,w,h);
  mpi->x=mpi->y=0;
  return mpi;
}

void mpcodecs_draw_slice(sh_video_t *sh, mp_image_t*mpi) {
  struct vf_instance_s* vf = sh->vfilter;
  vf->put_slice(vf,mpi);
}
