/*
   HACKING notes:
   first time it was OpenDivx project by Mayo (unsupported by mplayerxp)
   second it was divx4linux-beta by divx networks
   third it was divx4linux by divx network
   last it became divx5 by divx networks
   All these libraries have the same name libdivxdecore.so :(
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>

#include "config.h"
#include "help_mp.h"

#include <dlfcn.h> /* GLIBC specific. Exists under cygwin too! */

#include "vd_internal.h"
#include "codecs_ld.h"
#include "libvo/video_out.h"

static vd_info_t info = {
	"DivX4Linux lib (divx4/5 mode)",
	"divx4",
	"Nick Kurshev",
	"http://www.divx.com",
	"native codecs"
};

LIBVD_EXTERN(divx4)

#include "interface/divx4linux.h"

#define DIVX4LINUX_BETA 0
#define DIVX4LINUX	1
#define DIVX5LINUX	2

static int dll_ver=DIVX4LINUX_BETA;
static unsigned char d4id[35] = 
{
 0x83, 0xEC, 0x2C, 0x55, 0x57, 0x56, 0x53, 0xE8, 0x00, 0x00, 0x00, 0x00, 0x5B,
 0x81, 0xC3, 0xD8, 0xC9, 0x03, 0x00, 0x8B, 0x74, 0x24, 0x40, 0x8B, 0x4C, 0x24,
 0x44, 0x8B, 0x6C, 0x24, 0x48, 0x8B, 0x7C, 0x24, 0x4C
};
static int (STDCALL *decore_ptr)(unsigned long handle,
			unsigned long dec_opt,
			void *param1,
			void *param2);
static void *dll_handle;

// to set/get/query special features/parameters
static int control(sh_video_t *sh,int cmd,void* arg,...){
    switch(cmd){
    case VDCTRL_QUERY_MAX_PP_LEVEL:
	return 9; // for divx4linux
    case VDCTRL_SET_PP_LEVEL: {
	DEC_SET dec_set;
	int quality=*((int*)arg);
	if(quality<0 || quality>9) quality=9;
	dec_set.postproc_level=quality*10;
	(*decore_ptr)(0x123,DEC_OPT_SETPP,&dec_set,NULL);
	return CONTROL_OK;
    }
    case VDCTRL_SET_EQUALIZER: {
	if(dll_ver > DIVX4LINUX_BETA)
	{
	va_list ap;
	int value;
        int option;
	va_start(ap, arg);
	value=va_arg(ap, int);
	va_end(ap);

        if(!strcmp(arg,VO_EC_BRIGHTNESS)) option=DEC_GAMMA_BRIGHTNESS;
        else if(!strcmp(arg, VO_EC_CONTRAST)) option=DEC_GAMMA_CONTRAST;
        else if(!strcmp(arg,VO_EC_SATURATION)) option=DEC_GAMMA_SATURATION;
        else return CONTROL_FALSE;
	
        value = (value * 256) / 100;
        (*decore_ptr)(0x123, DEC_OPT_GAMMA, (void *)option, (void *) value);
	return CONTROL_OK;
	}
    }
    }

    return CONTROL_UNKNOWN;
}


static int load_lib( const char *libname )
{
  void *symbol;
  if(!(dll_handle=ld_codec(libname,"http://www.divx.com/divx/maclinux.php"))) return 0;
  if(ld_sym(dll_handle,"clp"))
  {
    MSG_ERR("vd_divx4: open divx detected: it's unsupported\n");    
    dlclose(dll_handle);
    return 0;
  }
  decore_ptr = ld_sym(dll_handle,"decore");
  if(memcmp(decore_ptr,d4id,35) == 0) dll_ver = DIVX4LINUX;
  symbol = ld_sym(dll_handle,"qpel_CopyMBlock");
  if(symbol) dll_ver = DIVX5LINUX;
  return decore_ptr != NULL;
}

// init driver
static int init(sh_video_t *sh){
    DEC_PARAM dec4_param;
    DEC5_PARAM dec5_param;
    DEC_PARAM * dec_param;
    DEC_SET dec_set;
    int bits=16;
    if(!load_lib("libdivxdecore"SLIBSUFFIX))    
	if(!load_lib("libdivxdecore"SLIBSUFFIX".0"))
    	    if(!load_lib("libdivxdecore.0")) return 0;
    MSG_INFO("vd_divx4: Found decore ver. %s\n",
	    dll_ver > DIVX4LINUX_BETA ? dll_ver > DIVX4LINUX ? "divx5linux" : "divx4linux" : "divx4linux.beta");
    dec_param = dll_ver > DIVX4LINUX ? &dec5_param : &dec4_param;
    if(dll_ver < DIVX5LINUX && sh->format==mmioFOURCC('D','X','5','0')){
	MSG_INFO("DivX 5.00 not supported by divx4linux decore - it requires divx5linux\n");
	return 0; // not supported
    }
    memset(dec_param,0,sizeof(dec4_param));
    if(!(mpcodecs_config_vo(sh,sh->disp_w,sh->disp_h,NULL))) return 0;
    switch(sh->codec->outfmt[sh->outfmtidx]){
	case IMGFMT_YV12: dec_param->output_format=DEC_YV12;bits=12;break;
	case IMGFMT_YUY2: dec_param->output_format=DEC_YUY2;break;
	case IMGFMT_UYVY: dec_param->output_format=DEC_UYVY;break;
	case IMGFMT_I420: dec_param->output_format=DEC_420;bits=12;break;
	case IMGFMT_BGR15: dec_param->output_format=
			   dll_ver > DIVX4LINUX ? DEC5_RGB555_INV : DEC_RGB555_INV;
			   break;
	case IMGFMT_BGR16: dec_param->output_format=
			   dll_ver > DIVX4LINUX ? DEC5_RGB565_INV : DEC_RGB565_INV;
			   break;
	case IMGFMT_BGR24: dec_param->output_format=
			   dll_ver > DIVX4LINUX ? DEC5_RGB24_INV : DEC_RGB24_INV;
			   bits=24;break;
	case IMGFMT_BGR32: dec_param->output_format=
			   dll_ver > DIVX4LINUX ? DEC5_RGB32_INV : DEC_RGB32_INV;
			   bits=32;
			   break;
	default:
	  MSG_ERR("Unsupported out_fmt: 0x%X\n",sh->codec->outfmt[sh->outfmtidx]);
	  return 0;
    }
    if(dll_ver > DIVX4LINUX)
    {
    switch(sh->format) {
      case mmioFOURCC('D','I','V','3'):
       	dec5_param.codec_version = 311;
	break;
      case mmioFOURCC('D','I','V','X'):
       	dec5_param.codec_version = 400;
	break;
      case mmioFOURCC('D','X','5','0'):
      default: // Fallback to DivX 5 behaviour
       	dec5_param.codec_version = 500;
    }
    dec5_param.build_number = 0;
    }
    dec_param->x_dim = sh->disp_w;
    dec_param->y_dim = sh->disp_h;
    (*decore_ptr)(0x123, DEC_OPT_INIT, dec_param, NULL);

    dec_set.postproc_level = divx_quality;
    (*decore_ptr)(0x123, DEC_OPT_SETPP, &dec_set, NULL);
    
    MSG_V("INFO: DivX4Linux (libdivxdecore.0) video codec init OK!\n");
    fflush(stdout);
    return 1;
}

// uninit driver
static void uninit(sh_video_t *sh){
    (*decore_ptr)(0x123,DEC_OPT_RELEASE,NULL,NULL);
    dlclose(dll_handle);
}

// decode a frame
static mp_image_t* decode(sh_video_t *sh,void* data,int len,int flags){
    mp_image_t* mpi;
    DEC_FRAME dec_frame;

    if(len<=0) return NULL; // skipped frame

    dec_frame.length = len;
    dec_frame.bitstream = data;
    dec_frame.render_flag = (flags&3)?0:1;

    mpi=mpcodecs_get_image(sh, MP_IMGTYPE_TEMP, MP_IMGFLAG_PRESERVE | MP_IMGFLAG_ACCEPT_WIDTH,
	0,0,sh->disp_w, sh->disp_h);
    if(mpi->flags&MP_IMGFLAG_DIRECT) mpi->flags|=MP_IMGFLAG_RENDERED;

    dec_frame.bmp=mpi->planes[0];
    dec_frame.stride=(mpi->flags&(MP_IMGFLAG_YUV|MP_IMGFLAG_DIRECT))==(MP_IMGFLAG_YUV|MP_IMGFLAG_DIRECT)?
		     mpi->flags&MP_IMGFLAG_PLANAR?mpi->stride[0]:mpi->stride[0]/2:
		     mpi->width;
    (*decore_ptr)(0x123, dll_ver>DIVX4LINUX?DEC_OPT_FRAME:(sh->format==mmioFOURCC('D','I','V','3'))?DEC_OPT_FRAME_311:DEC_OPT_FRAME, &dec_frame, NULL);

    return mpi;
}

