/*
   Xvid codec - is successor of odivx
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>

#include "config.h"
#include "help_mp.h"

#include <dlfcn.h> /* GLIBC specific. Exists under cygwin too! */

#include "vd_internal.h"
#include "codecs_ld.h"

static vd_info_t info = {
	"XviD MPEG4 codec ",
	"xvid",
	"Nick Kurshev",
	"http://www.xvid.org <(C) Christoph Lampert (gruel@web.de)>",
	"native codecs"
};

LIBVD_EXTERN(xvid)

#include "interface/divx4linux.h"

static int (STDCALL *decore_ptr)(unsigned long handle,
			unsigned long dec_opt,
			void *param1,
			void *param2);
static void *dll_handle;

// to set/get/query special features/parameters
static int control(sh_video_t *sh,int cmd,void* arg,...){
    return CONTROL_UNKNOWN;
}


static int load_lib( const char *libname )
{
  if(!(dll_handle=ld_codec(libname,"http://www.xvid.org"))) return 0;
  decore_ptr = ld_sym(dll_handle,"decore");
  return decore_ptr != NULL;
}

// init driver
static int init(sh_video_t *sh){
    DEC_PARAM dec_param;
    if(!load_lib("libxvidcore"SLIBSUFFIX)) return 0;
//    if(!load_lib("libdivxdecore.0")) return 0;
    memset(&dec_param,0,sizeof(dec_param));
    if(!(mpcodecs_config_vo(sh,sh->disp_w,sh->disp_h,NULL))) return 0;
    switch(sh->codec->outfmt[sh->outfmtidx]){
	case IMGFMT_YV12: dec_param.output_format=DEC_YV12;break;
	case IMGFMT_YUY2: dec_param.output_format=DEC_YUY2;break;
	case IMGFMT_UYVY: dec_param.output_format=DEC_UYVY;break;
	case IMGFMT_I420: dec_param.output_format=DEC_420;break;
	case IMGFMT_BGR15:dec_param.output_format=DEC_RGB555_INV;break;
	case IMGFMT_BGR16: dec_param.output_format=DEC_RGB565_INV;break;
	case IMGFMT_BGR24: dec_param.output_format=DEC_RGB24_INV;break;
	case IMGFMT_BGR32: dec_param.output_format=DEC_RGB32_INV;break;
	default:
	  MSG_ERR("Unsupported out_fmt: 0x%X\n",sh->codec->outfmt[sh->outfmtidx]);
	  return 0;
    }
    dec_param.x_dim = sh->disp_w;
    dec_param.y_dim = sh->disp_h;
    (*decore_ptr)(0x123, DEC_OPT_INIT, &dec_param, NULL);
    
    MSG_V("INFO: DivX4Linux (libdivxdecore.0) video codec init OK!\n");
    fflush(stdout);
    return 1;
}

// uninit driver
static void uninit(sh_video_t *sh){
    (*decore_ptr)(0x123,DEC_OPT_RELEASE,NULL,NULL);
    dlclose(dll_handle);
}

// decode a frame
static mp_image_t* decode(sh_video_t *sh,void* data,int len,int flags){
    mp_image_t* mpi;
    DEC_FRAME dec_frame;

    if(len<=0) return NULL; // skipped frame

    dec_frame.length = len;
    dec_frame.bitstream = data;
    dec_frame.render_flag = (flags&3)?0:1;

    mpi=mpcodecs_get_image(sh, MP_IMGTYPE_TEMP, MP_IMGFLAG_PRESERVE | MP_IMGFLAG_ACCEPT_WIDTH,
	0,0,sh->disp_w, sh->disp_h);
    if(mpi->flags&MP_IMGFLAG_DIRECT) mpi->flags|=MP_IMGFLAG_RENDERED;

    dec_frame.bmp=mpi->planes[0];
    dec_frame.stride=(mpi->flags&(MP_IMGFLAG_YUV|MP_IMGFLAG_DIRECT))==(MP_IMGFLAG_YUV|MP_IMGFLAG_DIRECT)?
		     mpi->flags&MP_IMGFLAG_PLANAR?mpi->stride[0]:mpi->stride[0]/2:
		     mpi->width;
    (*decore_ptr)(0x123, (sh->format==mmioFOURCC('D','I','V','3'))?DEC_OPT_FRAME_311:DEC_OPT_FRAME, &dec_frame, NULL);

    return mpi;
}

