#define OSD_SUPPORT

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../config.h"
#include "pp_msg.h"

#include "../libvo/img_format.h"
#include "mp_image.h"
#include "vf.h"

#include "../libvo/fastmemcpy.h"

#ifdef OSD_SUPPORT
#include "../libvo/sub.h"
#include "../libvo/osd.h"
#endif

#define MAX(a,b) ((a) > (b) ? (a) : (b))

struct vf_priv_s {
    unsigned org_w,w;
    unsigned org_h,h;
    float panscan;
};

//===========================================================================//

static int __FASTCALL__ config(struct vf_instance_s* vf,
        int width, int height, int d_width, int d_height,
	unsigned int flags, unsigned int outfmt,void *tune){
    unsigned w,h,d_w,d_h;
    vf->priv->org_w=width;
    vf->priv->org_h=height;
    if(vf->priv->panscan>=1.)
    {
	w=(float)width/vf->priv->panscan;
	d_w = (float)d_width/vf->priv->panscan;
	if(outfmt==IMGFMT_YV12 || outfmt==IMGFMT_I420)
	{
		w&=~3;
		d_w&=~3;
	}
	else
	if(outfmt==IMGFMT_YVU9 || outfmt==IMGFMT_IF09)
	{
		w&=~7;
		d_w&=~7;
	}
	w=(float)width/vf->priv->panscan;
	vf->priv->h=h=height;
	d_h = d_height;
    }
    else
    {
	vf->priv->w=w=width;
	d_w = d_width;
	h=(float)height*vf->priv->panscan;
	d_h = (float)d_height*vf->priv->panscan;
	if(outfmt==IMGFMT_YV12 || outfmt==IMGFMT_I420)
	{
		h&=~3;
		d_h&=~3;
	}
	else
	if(outfmt==IMGFMT_YVU9 || outfmt==IMGFMT_IF09)
	{
		h&=~7;
		d_h&=~7;
	}
	vf->priv->h=h;
    }
    MSG_DBG2("w,h=%i %i org_w,h=%i %i\n",w,h,vf->priv->org_w,vf->priv->org_h);
    return vf_next_config(vf,w,h,d_w,d_h,flags,outfmt,tune);
}

static int __FASTCALL__ put_slice(struct vf_instance_s* vf, mp_image_t *mpi){

    mp_image_t *dmpi;
    unsigned w,dh,ph;
    int dy,py;
    int ps_x = (vf->priv->org_w - vf->priv->w)>>1;
    int ps_y = (vf->priv->org_h - vf->priv->h)>>1;
    w=mpi->w>vf->priv->w?vf->priv->w:mpi->w;
    ph=vf->priv->h+ps_y;
    /* In hope that mpi->x === 0 */
    MSG_DBG2("%i %i %i %i ps %i %i\n",mpi->x,mpi->y,mpi->w,mpi->h,ps_x,ps_y);
    if(mpi->y+mpi->h < ps_y || mpi->y>ph) return 1; /* nothing todo */
    dh  = mpi->h;
    if(mpi->y < ps_y) dh = mpi->h+mpi->y-ps_y;
    if(mpi->y+mpi->h > ph) dh = ph-mpi->y;
    py=mpi->y-ps_y;
    dy=0;
    if(py<0) { dy=-py; py=0; }
    dmpi=vf_get_slice(vf->next,mpi->imgfmt,
	    MP_IMGTYPE_TEMP, MP_IMGFLAG_ACCEPT_STRIDE,
	    mpi->x,py,w,dh);
    MSG_DBG2("py=%i dy=%i dh=%i dmpi %X %X %X\n",py,dy,dh,dmpi->planes[0],dmpi->planes[1],dmpi->planes[2]);

    if(mpi->flags&MP_IMGFLAG_PLANAR){
	memcpy_pic(dmpi->planes[0],
		mpi->planes[0]+ps_x+dy*mpi->stride[0], w, dh,
		dmpi->stride[0],mpi->stride[0]);

	memcpy_pic(dmpi->planes[1],
		mpi->planes[1]+(ps_x>>mpi->chroma_x_shift)+(dy>>mpi->chroma_y_shift)*mpi->stride[1],
		w>>mpi->chroma_x_shift,
		dh>>mpi->chroma_y_shift,
		dmpi->stride[1],mpi->stride[1]);
	memcpy_pic(dmpi->planes[2],
		mpi->planes[2]+(ps_x>>mpi->chroma_x_shift)+(dy>>mpi->chroma_y_shift)*mpi->stride[2],
		w>>mpi->chroma_x_shift,
		dh>>mpi->chroma_y_shift,
		dmpi->stride[2],mpi->stride[2]);

    } else {
	memcpy_pic(dmpi->planes[0],
		mpi->planes[0]+ps_x*(mpi->bpp/8)+dy*mpi->stride[0], w*(mpi->bpp/8), dh,
		dmpi->stride[0],mpi->stride[0]);
	dmpi->planes[1] = mpi->planes[1]; // passthrough rgb8 palette
    }
    return vf_next_put_slice(vf,dmpi);
}

//===========================================================================//

static int __FASTCALL__ control(struct vf_instance_s* vf, int request, void* data){
    return vf_next_control(vf,request,data);
}

static int __FASTCALL__ query_format(struct vf_instance_s* vf, unsigned int fmt,unsigned w,unsigned h){
    return vf_next_query_format(vf->next,fmt,w,h);
}

static int __FASTCALL__ vf_open(vf_instance_t *vf, char* args){
    vf->config=config;
    vf->control=control;
    vf->query_format=query_format;
    vf->put_slice=put_slice;
    if(!vf->priv) vf->priv=malloc(sizeof(struct vf_priv_s));
    memset(vf->priv,0,sizeof(struct vf_priv_s));
    vf->priv->panscan=0;
    if(args) sscanf(args,"%f",&vf->priv->panscan);
    if(vf->priv->panscan<=0.) return 0;
    return 1;
}

vf_info_t vf_info_panscan = {
    "Panning scan",
    "panscan",
    "Nick Kurshev",
    "",
    vf_open
};

//===========================================================================//
